@@ -28,32 +28,30 @@
 #include "str.h"
 #include "utils.h"
 
-int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int timeout)
+int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, int timeout)
 {
-	char *buffer = mymalloc(128, "http header");
-	int len_in=0, len=128;
-	char byte=-1, prev_byte=-2;
+	int len_in=0, len=4096;
+	char *buffer = mymalloc(len, "http header");
 	int rc = RC_OK;
 
 	*headers = NULL;
 
+	memset(buffer, 0x00, len);
+
 	for(;;)
 	{
 		int rrc;
-
-		if (byte != 13)
-			prev_byte = byte;
+		int now_n = (len - len_in) - 1;
 
 #ifndef NO_SSL
 		if (ssl_h)
-			rrc = READ_SSL(ssl_h, &byte, 1);
+			rrc = SSL_read(ssl_h, &buffer[len_in], now_n);
 		else
 #endif
-			rrc = myread(socket_h, &byte, 1, timeout);
+			rrc = read_to(socket_h, &buffer[len_in], now_n, timeout);
 		if (rrc == 0 || rrc == RC_SHORTREAD)	/* socket closed before request was read? */
 		{
 			rc = RC_SHORTREAD;
-			buffer[len_in] = 0x00;
 			break;
 		}
 		else if (rrc == RC_TIMEOUT)		/* timeout */
@@ -61,22 +59,21 @@ int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int timeout)
 			return RC_TIMEOUT;
 		}
 
-		if (len_in >= (len-1))
+		len_in += rrc;
+
+		buffer[len_in] = 0x00;
+		if (strstr(buffer, "\r\n\r\n") != NULL)
+			break;
+
+		if (len_in == (len - 1))
 		{
 			len <<= 1;
 			buffer = (char *)myrealloc(buffer, len, "http reply");
 		}
-
-		buffer[len_in++] = byte;
-
-		if (prev_byte == 10 && byte == 10)
-		{
-			buffer[len_in]=0x00;
-			break;	
-		}
 	}
 
 	*headers = buffer;
+	*overflow = len_in - (strstr(buffer, "\r\n\r\n") - buffer + 4);
 
 	return rc;
 }
