@@ -13,4 +13,4 @@
   files in the program, then also delete it here.
 */
 
-int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int timeout);
+int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, int timeout);
