@@ -27,6 +27,36 @@
 
 extern char last_error[];
 
+ssize_t read_to(int fd, char *whereto, size_t len, int timeout)
+{
+	for(;;)
+	{
+		ssize_t rc;
+		struct timeval to;
+		fd_set rfds;
+
+		FD_ZERO(&rfds);
+		FD_SET(fd, &rfds);
+
+		to.tv_sec  = timeout / 1000;
+		to.tv_usec = (timeout - (to.tv_sec * 1000)) * 1000;
+
+		rc = select(fd + 1, &rfds, NULL, NULL, &to);
+		if (rc == 0)
+			return RC_TIMEOUT;
+		else if (rc == -1)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+
+			snprintf(last_error, ERROR_BUFFER_SIZE, "myread::select failed: %s\n", strerror(errno));
+			return RC_SHORTREAD;
+		}
+
+		return read(fd, whereto, len);
+	}
+}
+
 ssize_t myread(int fd, char *whereto, size_t len, int timeout)
 {
 	ssize_t cnt=0;
