@@ -52,7 +52,7 @@ char last_error[ERROR_BUFFER_SIZE];
 
 void version(void)
 {
-	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2010 folkert@vanheusden.com\n");
+	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2011 folkert@vanheusden.com\n");
 #ifndef NO_SSL
 	fprintf(stderr, "SSL support included\n");
 #endif
@@ -65,8 +65,10 @@ void usage(void)
 	fprintf(stderr, "-p portnr      portnumber (e.g. 80)\n");
 	fprintf(stderr, "-x host:port   hostname+portnumber of proxyserver\n");
 	fprintf(stderr, "-c count       how many times to connect\n");
-	fprintf(stderr, "-i interval    delay between each connect\n");
+	fprintf(stderr, "-i interval    delay between each connect, can be only smaller than 1 if user is root\n");
 	fprintf(stderr, "-t timeout     timeout (default: 30s)\n");
+	fprintf(stderr, "-Z             ask any proxies on the way not to cache the requests\n");
+	fprintf(stderr, "-Q             use a persistent connection. adds a 'C' to the output if httping had to reconnect\n");
 	fprintf(stderr, "-6             use IPv6\n");
 	fprintf(stderr, "-s             show statuscodes\n");
 	fprintf(stderr, "-S             split time in connect-time and processing time\n");
@@ -143,6 +145,8 @@ void usage(void)
 	fprintf(stderr, "--username		-U\n");
 	fprintf(stderr, "--password		-P\n");
 	fprintf(stderr, "--cookie		-C\n");
+	fprintf(stderr, "--persistent-connections	-Q\n");
+	fprintf(stderr, "--no-cache		-Z\n");
 	fprintf(stderr, "--version		-V\n");
 	fprintf(stderr, "--help			-h\n");
 }
@@ -156,10 +160,14 @@ void emit_error()
 
 	if (!nagios_mode)
 		last_error[0] = 0x00;
+
+	fflush(NULL);
 }
 
 void handler(int sig)
 {
+	fprintf(stderr, "Got signal %d\n", sig);
+
 	stop = 1;
 }
 
@@ -217,7 +225,7 @@ int main(int argc, char *argv[])
 	int req_len;
 	int c;
 	int count = -1, curncount = 0;
-	int wait = 1;
+	double wait = 1.0;
 	int audible = 0;
 	int ok = 0, err = 0;
 	double min = 999999999999999.0, avg = 0.0, max = 0.0;
@@ -253,9 +261,13 @@ int main(int argc, char *argv[])
 	int fd = -1;
 	SSL *ssl_h = NULL;
 	BIO *s_bio;
-	struct sockaddr_in6 bind_to;
+	struct sockaddr_in *bind_to = NULL;
+	struct sockaddr_in bind_to_4;
+	struct sockaddr_in6 bind_to_6;
 	char bind_to_valid = 0;
 	char split = 0, use_ipv6 = 0;
+	char persistent_connections = 0, persistent_did_reconnect = 0;
+	char no_cache = 0;
 
 	static struct option long_options[] =
 	{
@@ -264,6 +276,7 @@ int main(int argc, char *argv[])
 		{"port",	1, NULL, 'p' },
 		{"host-port",	1, NULL, 'x' },
 		{"count",	1, NULL, 'c' },
+		{"persistent-connections",	0, NULL, 'Q' },
 		{"interval",	1, NULL, 'i' },
 		{"timeout",	1, NULL, 't' },
 		{"ipv6",	0, NULL, '6' },
@@ -274,6 +287,7 @@ int main(int argc, char *argv[])
 		{"show-xfer-speed-compressed",	0, NULL, 'B' },
 		{"data-limit",	1, NULL, 'L' },
 		{"show-kb",	0, NULL, 'X' },
+		{"no-cache",	0, NULL, 'Z' },
 #ifndef NO_SSL
 		{"use-ssl",	0, NULL, 'l' },
 		{"show-fingerprint",	0, NULL, 'z' },
@@ -306,10 +320,14 @@ int main(int argc, char *argv[])
 
 	buffer = (char *)mymalloc(page_size, "receive buffer");
 
-	while((c = getopt_long(argc, argv, "6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z:AP:U:C:", long_options, NULL)) != -1)
+	while((c = getopt_long(argc, argv, "ZQ6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z:AP:U:C:", long_options, NULL)) != -1)
 	{
 		switch(c)
 		{
+			case 'Z':
+				no_cache = 1;
+				break;
+
 			case '6':
 				use_ipv6 = 1;
 				break;
@@ -318,23 +336,37 @@ int main(int argc, char *argv[])
 				split = 1;
 				break;
 
+			case 'Q':
+				persistent_connections = 1;
+				break;
+
 			case 'y':
 				{
 					char *dummy = strchr(optarg, ':');
 
 					bind_to_valid = 1;
 
-					memset(&bind_to, 0x00, sizeof(bind_to));
-
 					if (dummy)
 					{
-						bind_to.sin6_port = htons(atoi(dummy + 1));
-						*dummy = 0x00;
-					}
+						bind_to = (struct sockaddr_in *)&bind_to_6;
+						memset(&bind_to_6, 0x00, sizeof(bind_to_6));
+						bind_to_6.sin6_family = AF_INET6;
 
-					if (inet_pton(AF_INET6, optarg, &(bind_to.sin6_addr)) != 1)
+						if (inet_pton(AF_INET6, optarg, &(bind_to_6.sin6_addr)) != 1)
+						{
+							error_exit("cannot convert ip address '%s' (for -y)\n", optarg);
+						}
+					}
+					else
 					{
-						error_exit("cannot convert ip address '%s' (for -y)\n", optarg);
+						bind_to = (struct sockaddr_in *)&bind_to_4;
+						memset(&bind_to_4, 0x00, sizeof(bind_to_4));
+						bind_to_4.sin_family = AF_INET;
+
+						if (inet_pton(AF_INET, optarg, &(bind_to_4.sin_addr)) != 1)
+						{
+							error_exit("cannot convert ip address '%s' (for -y)\n", optarg);
+						}
 					}
 				}
 				break;
@@ -393,7 +425,12 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'i':
-				wait = atoi(optarg);
+				wait = atof(optarg);
+				if (wait < 1.0 && getuid() != 0)
+				{
+					fprintf(stderr, "Only root can use intervals smaller than 1\n");
+					wait = 1.0;
+				}
 				break;
 
 			case 't':
@@ -479,12 +516,8 @@ int main(int argc, char *argv[])
 	if (optind < argc)
 		get = argv[optind];
 
-
 	last_error[0] = 0x00;
 
-
-       
-
 	if (!get_instead_of_head && show_Bps)
 		error_exit("-b/-B can only be used when also using -G\n");
 
@@ -504,15 +537,15 @@ int main(int argc, char *argv[])
 		}
 
 		/*
-		if (strncasecmp(getcopy, http_string, http_string_len) != 0)
-		{
-			fprintf(stderr, "'%s' is a strange URL\n", getcopy);
-			fprintf(stderr, "Expected: %s...\n", http_string);
-			if (strncasecmp(getcopy, "https://", 8) == 0)
-				fprintf(stderr, "Did you forget to add the '-l' switch to the httping commandline?\n");
-			return 2;
-		}
-		*/
+		   if (strncasecmp(getcopy, http_string, http_string_len) != 0)
+		   {
+		   fprintf(stderr, "'%s' is a strange URL\n", getcopy);
+		   fprintf(stderr, "Expected: %s...\n", http_string);
+		   if (strncasecmp(getcopy, "https://", 8) == 0)
+		   fprintf(stderr, "Did you forget to add the '-l' switch to the httping commandline?\n");
+		   return 2;
+		   }
+		 */
 
 		slash = strchr(getcopy, '/');
 		if (slash)
@@ -586,9 +619,9 @@ int main(int argc, char *argv[])
 	}
 #endif
 
-	request = mymalloc(strlen(get) + 4096, "request");
+	request = mymalloc(strlen(get) + 8192, "request");
 	if (proxyhost)
-		sprintf(request, "%s %s HTTP/1.0\r\n", get_instead_of_head?"GET":"HEAD", get);
+		sprintf(request, "%s %s HTTP/1.%c\r\n", get_instead_of_head?"GET":"HEAD", get, persistent_connections?'1':'0');
 	else
 	{
 		char *dummy = get, *slash;
@@ -599,9 +632,9 @@ int main(int argc, char *argv[])
 
 		slash = strchr(dummy, '/');
 		if (slash)
-			sprintf(request, "%s %s HTTP/1.0\r\n", get_instead_of_head?"GET":"HEAD", slash);
+			sprintf(request, "%s %s HTTP/1.%c\r\n", get_instead_of_head?"GET":"HEAD", slash, persistent_connections?'1':'0');
 		else
-			sprintf(request, "%s / HTTP/1.0\r\n", get_instead_of_head?"GET":"HEAD");
+			sprintf(request, "%s / HTTP/1.%c\r\n", get_instead_of_head?"GET":"HEAD", persistent_connections?'1':'0');
 	}
 	if (useragent)
 		sprintf(&request[strlen(request)], "User-Agent: %s\r\n", useragent);
@@ -613,6 +646,12 @@ int main(int argc, char *argv[])
 	if (ask_compression)
 		sprintf(&request[strlen(request)], "Accept-Encoding: gzip,deflate\r\n");
 
+	if (no_cache)
+	{
+		sprintf(&request[strlen(request)], "Pragma: no-cache\r\n");
+		sprintf(&request[strlen(request)], "Cache-Control: no-cache\r\n");
+	}
+
 	/* Basic Authentification */
 	if (auth_mode) { 
 		char auth_string[255];
@@ -629,6 +668,9 @@ int main(int argc, char *argv[])
 		sprintf(&request[strlen(request)], "Cookie: %s;\r\n", cookie);
 	}
 
+	if (persistent_connections)
+		sprintf(&request[strlen(request)], "Connection: keep-alive\r\n");
+
 	strcat(request, "\r\n");
 	req_len = strlen(request);
 
@@ -655,6 +697,9 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if (persistent_connections)
+		fd = -1;
+
 	while((curncount < count || count == -1) && stop == 0)
 	{
 		double ms;
@@ -671,10 +716,13 @@ int main(int argc, char *argv[])
 			char *fp = NULL;
 			int rc;
 			char *sc = NULL, *scdummy = NULL;
+			int persistent_tries = 0;
+			int len = 0, overflow = 0, headers_len;
 
 			curncount++;
 
-			if (!resolve_once || (resolve_once == 1 && have_resolved == 0))
+persistent_loop:
+			if ((!resolve_once || (resolve_once == 1 && have_resolved == 0)) && fd == -1)
 			{
 				memset(&addr, 0x00, sizeof(addr));
 
@@ -688,26 +736,38 @@ int main(int argc, char *argv[])
 				have_resolved = 1;
 			}
 
-			fd = connect_to(bind_to_valid?(struct sockaddr *)&bind_to:NULL, &ai, timeout);
+			if ((persistent_connections && fd < 0) || (!persistent_connections))
+				fd = connect_to(bind_to_valid?bind_to:NULL, &ai, timeout);
 
 			if (fd == -3)	/* ^C pressed */
 				break;
 
 			if (fd < 0)
+			{
 				emit_error();
+				fd = -1;
+			}
 
 			if (fd >= 0)
 			{
 				/* set socket to low latency */
 				if (set_tcp_low_latency(fd) == -1)
+				{
+					close(fd);
+					fd = -1;
 					break;
+				}
 
 				/* set fd blocking */
 				if (set_fd_blocking(fd) == -1)
+				{
+					close(fd);
+					fd = -1;
 					break;
+				}
 
 #ifndef NO_SSL
-				if (use_ssl)
+				if (use_ssl && ssl_h == NULL)
 				{
 					int rc;
 
@@ -716,6 +776,17 @@ int main(int argc, char *argv[])
 					{
 						close(fd);
 						fd = rc;
+
+						if (persistent_connections)
+						{
+							if (++persistent_tries < 2)
+							{
+								close(fd);
+								fd = -1;
+								persistent_did_reconnect = 1;
+								goto persistent_loop;
+							}
+						}
 					}
 				}
 #endif
@@ -729,9 +800,10 @@ int main(int argc, char *argv[])
 					snprintf(last_error, ERROR_BUFFER_SIZE, "timeout connecting to host\n");
 
 				emit_error();
-
 				err++;
 
+				fd = -1;
+
 				break;
 			}
 
@@ -743,6 +815,17 @@ int main(int argc, char *argv[])
 				rc = mywrite(fd, request, req_len, timeout);
 			if (rc != req_len)
 			{
+				if (persistent_connections)
+				{
+					if (++persistent_tries < 2)
+					{
+						close(fd);
+						fd = -1;
+						persistent_did_reconnect = 1;
+						goto persistent_loop;
+					}
+				}
+
 				if (rc == -1)
 					snprintf(last_error, ERROR_BUFFER_SIZE, "error sending request to host\n");
 				else if (rc == -2)
@@ -755,12 +838,13 @@ int main(int argc, char *argv[])
 				emit_error();
 
 				close(fd);
+				fd = -1;
 				err++;
 
 				break;
 			}
 
-			rc = get_HTTP_headers(fd, ssl_h, &reply, timeout);
+			rc = get_HTTP_headers(fd, ssl_h, &reply, &overflow, timeout);
 
 			if ((show_statuscodes || machine_readable) && reply != NULL)
 			{
@@ -792,6 +876,8 @@ int main(int argc, char *argv[])
 				{
 					char *dummy = strchr(encoding + 1, '\n');
 					if (dummy) *dummy = 0x00;
+					dummy = strchr(sc, '\r');
+					if (dummy) *dummy = 0x00;
 
 					if (strstr(encoding, "gzip") == 0 || strstr(encoding, "deflate") == 0)
 					{
@@ -800,10 +886,37 @@ int main(int argc, char *argv[])
 				}
 			}
 
+			if (persistent_connections && show_bytes_xfer)
+			{
+				char *length = strstr(reply, "\nContent-Length:");
+				if (!length)
+				{
+					snprintf(last_error, ERROR_BUFFER_SIZE, "'Content-Length'-header missing!\n");
+					emit_error();
+					close(fd);
+					fd = -1;
+					break;
+				}
+				len = atoi(&length[17]);
+			}
+
+			headers_len = (strstr(reply, "\r\n\r\n") - reply) + 4;
+
 			free(reply);
 
 			if (rc < 0)
 			{
+				if (persistent_connections)
+				{
+					if (++persistent_tries < 2)
+					{
+						close(fd);
+						fd = -1;
+						persistent_did_reconnect = 1;
+						goto persistent_loop;
+					}
+				}
+
 				if (rc == RC_SHORTREAD)
 					snprintf(last_error, ERROR_BUFFER_SIZE, "error receiving reply from host\n");
 				else if (rc == RC_TIMEOUT)
@@ -812,6 +925,7 @@ int main(int argc, char *argv[])
 				emit_error();
 
 				close(fd);
+				fd = -1;
 				err++;
 
 				break;
@@ -819,13 +933,21 @@ int main(int argc, char *argv[])
 
 			ok++;
 
-			if (get_instead_of_head)
+			if (get_instead_of_head && show_Bps)
 			{
 				double dl_start = get_ts(), dl_end;
+				int cur_limit = Bps_limit;
+
+				if (persistent_connections)
+				{
+					if (cur_limit == -1 || len < cur_limit)
+						cur_limit = len - overflow;
+				}
 
 				for(;;)
 				{
-					int rc = read(fd, buffer, page_size);
+					int n = cur_limit != -1 ? min(cur_limit - bytes_transferred, page_size) : page_size;
+					int rc = read(fd, buffer, n);
 
 					if (rc == -1)
 					{
@@ -837,7 +959,7 @@ int main(int argc, char *argv[])
 
 					bytes_transferred += rc;
 
-					if (Bps_limit != -1 && bytes_transferred >= Bps_limit)
+					if (cur_limit != -1 && bytes_transferred >= cur_limit)
 						break;
 				}
 
@@ -852,7 +974,7 @@ int main(int argc, char *argv[])
 			dend = get_ts();
 
 #ifndef NO_SSL
-			if (use_ssl)
+			if (use_ssl && !persistent_connections)
 			{
 				if (show_fp && ssl_h != NULL)
 				{
@@ -866,9 +988,15 @@ int main(int argc, char *argv[])
 				}
 
 				SSL_free(ssl_h);
+				ssl_h = NULL;
 			}
 #endif
-			close(fd);
+
+			if (!persistent_connections)
+			{
+				close(fd);
+				fd = -1;
+			}
 
 			ms = (dend - dstart) * 1000.0;
 			avg += ms;
@@ -905,13 +1033,26 @@ int main(int argc, char *argv[])
 			}
 			else if (!quiet && !nagios_mode)
 			{
-				if (split)
-				{
-					printf("connected to %s:%d, seq=%d time=%.2f+%.2f=%.2f ms %s", hostname, portnr, curncount-1, (dafter_connect - dstart) * 1000.0, (dend - dafter_connect) * 1000.0, ms, sc?sc:"");
-				}
+				char current_host[1024];
+				char *operation = !persistent_connections ? "connected to" : "pinged host";
+
+				if (getnameinfo((const struct sockaddr *)&addr, sizeof(addr), current_host, sizeof(current_host), NULL, 0, NI_NUMERICHOST) == -1)
+					snprintf(current_host, sizeof(current_host), "getnameinfo() failed: %d", errno);
+
+				if (persistent_connections && show_bytes_xfer)
+					printf("%s %s:%d (%d/%d bytes), seq=%d ", operation, current_host, portnr, headers_len, len, curncount-1);
 				else
+					printf("%s %s:%d (%d bytes), seq=%d ", operation, current_host, portnr, headers_len, curncount-1);
+
+				if (split)
+					printf("time=%.2f+%.2f=%.2f ms %s", (dafter_connect - dstart) * 1000.0, (dend - dafter_connect) * 1000.0, ms, sc?sc:"");
+				else
+					printf("time=%.2f ms %s", ms, sc?sc:"");
+
+				if (persistent_did_reconnect)
 				{
-					printf("connected to %s:%d, seq=%d time=%.2f ms %s", hostname, portnr, curncount-1, ms, sc?sc:"");
+					printf(" C");
+					persistent_did_reconnect = 0;
 				}
 
 				if (show_Bps)
@@ -956,7 +1097,7 @@ int main(int argc, char *argv[])
 		}
 
 		if (curncount != count && !stop)
-			sleep(wait);
+			usleep((useconds_t)(wait * 1000000.0));
 	}
 
 	avg_httping_time = avg / (double)ok;
