@@ -25,7 +25,7 @@
 
 #include "mssl.h"
 
-extern char *last_error;
+extern char last_error[];
 BIO *bio_err=0;
 
 char close_ssl_connection(SSL *ssl_h, int socket_h)
