@@ -15,4 +15,4 @@
 
 #define incopy(a)       *((struct in_addr *)a)
 
-int resolve_host(char *host, struct sockaddr_in *addr);
+int resolve_host(char *host, struct sockaddr_in6 *addr, struct addrinfo *ai, char use_ipv6, int portnr);
