@@ -18,7 +18,10 @@
 #include "mem.h"
 #include "utils.h"
 
-#ifndef strndup
+/*
+Most unixes have this function already.
+
+#ifndef _GNU_SOURCE
 char *strndup(char *in, int size)
 {
 	char *out = mymalloc(size + 1, "strndup");
@@ -29,3 +32,4 @@ char *strndup(char *in, int size)
 	return out;
 }
 #endif
+*/
