@@ -15,6 +15,8 @@
 
 #include <string.h>
 
-#ifndef strndup
+/*
+#ifndef _GNU_SOURCE
 char *strndup(char *in, int size);
 #endif
+*/
