@@ -31,7 +31,7 @@
 
 extern char last_error[];
 
-int connect_to(struct sockaddr *bind_to, struct sockaddr_in *addr, int portnr, int timeout)
+int connect_to(struct sockaddr *bind_to, struct addrinfo *ai, int timeout)
 {
 	int     fd;
 	int 	rc;
@@ -39,7 +39,7 @@ int connect_to(struct sockaddr *bind_to, struct sockaddr_in *addr, int portnr, i
 	fd_set wfds;
 
 	/* create socket */
-	fd = socket(AF_INET, SOCK_STREAM, 0);
+	fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 	if (fd == -1)
 	{
 		snprintf(last_error, ERROR_BUFFER_SIZE, "problem creating socket (%s)", strerror(errno));
@@ -65,9 +65,6 @@ int connect_to(struct sockaddr *bind_to, struct sockaddr_in *addr, int portnr, i
 		}
 	}
 
-	/* initialize address structure */
-	addr -> sin_port   = htons(portnr);
-
 	/* make fd nonblocking */
 	if (set_fd_nonblocking(fd) == -1)
 		return -1;
@@ -80,7 +77,7 @@ int connect_to(struct sockaddr *bind_to, struct sockaddr_in *addr, int portnr, i
 	to.tv_usec = (timeout - (to.tv_sec * 1000)) * 1000;
 
 	/* connect to peer */
-	if (connect(fd, (struct sockaddr *)addr, sizeof(struct sockaddr)) == 0)
+	if (connect(fd, ai -> ai_addr, ai -> ai_addrlen) == 0)
 	{
 		/* connection made, return */
 		return fd;
